/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
@file rectify.cpp
@ingroup ExamplePrograms
@brief Rectifies a 2D plane from the input image.

@todo Detailed description should follow.

@section Usage
Once compiled, the program should be used with this command line:
\code
	./rectify --URL=<path or url to video>
\endcode

@param URL Video source identificator. See dynamic property URL for @ref QVMPlayerCamera.
@author PARP Research Group. University of Murcia, Spain.
*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <math.h>

#include <QDebug>
#include <QVector>
#include <QThread>

#include <qvcore/qvapplication.h>
#include <qvcameras/qvmplayercamera.h>
#include <qvgui/qvgui.h>

#include <qvdta/qvpolyline.h>
#include <qvdta/qvdta.h>

#include <TooN/TooN.h>
#include <TooN/numerics.h>	// Basic TooN library
#include <TooN/numhelpers.h>	// If you want a few extras
#include <TooN/SVD.h>		// If you want singular value decompositions
#include <TooN/LU.h>		// If you want lu triangular decompositions
#include <TooN/SymEigen.h>	// If you want Eigen decomposition of symmetric matrices

#define PI	3.1415926535

Q_DECLARE_METATYPE(Matrix<>);

/// \cond

void GetHotPoints(const QVImage<sFloat> cornerResponseImage, QList<QPoint> &hotPoints, uInt maxWindow)
	{
	const uInt rows = cornerResponseImage.getRows(), cols = cornerResponseImage.getCols();

	QVImage<uChar> binaryCornerImage(cols, rows);
	FilterLocalMax(cornerResponseImage, binaryCornerImage, maxWindow, maxWindow);

	QVIMAGE_INIT_READ(uChar,binaryCornerImage);
	for (uInt row = 0; row < binaryCornerImage.getRows(); row++)
		for (uInt col = 0; col < binaryCornerImage.getCols(); col++)
			if (QVIMAGE_PIXEL(binaryCornerImage, col, row,0))
				hotPoints.append(QPoint(col, row));
	}

void GetMaximalPoints(const QVImage<sFloat> cornerResponseImage, QList<QPoint> &hotPoints, QList<QPoint> &maximalPoints, uInt maxPoints)
	{
	while( hotPoints.size() > 0 && maximalPoints.size() < maxPoints )
		{
		uInt maxIndex = 0;
		for (int n=0; n < hotPoints.size(); n++)
			if ( cornerResponseImage(hotPoints.at(n)) >  cornerResponseImage(hotPoints.at(maxIndex)) )
				maxIndex = n;

		maximalPoints.append(hotPoints.at(maxIndex));
		hotPoints.removeAt(maxIndex);
		}
	}

uInt	getClosestPointIndex(const QPoint point, const QList<QPoint> &pointList)
	{
	uInt index = 0;
	for (uInt n = 1; n < pointList.size(); n++)
		if ((point - pointList.at(n)).manhattanLength() < (point - pointList.at(index)).manhattanLength())
			index = n;

	return index;
	}

QPoint	getMeanPoint(const QList<QPoint> &pointList)
	{
	QPoint center(0,0);

	for (uInt n = 0; n < pointList.size(); n++)
		{
		center.rx() += pointList.at(n).x();
		center.ry() += pointList.at(n).y();
		}

	center.rx() = center.rx() / pointList.size();
	center.ry() = center.ry() / pointList.size();

	return center;
	}

double	angle(const QPoint &p)
	{
	double x = p.x(), y = p.y();
	if (x>0)
		if (y>=0)
			return atan(y/x);
		else
			return atan(y/x) + 2*PI;
	else if (x == 0)
		if (y>0)
			return PI/2;
		else
			return 3*PI/2;
	else // x < 0
		return atan(y/x)+PI;
	}

double	clockWiseAngle(const QPoint &p1, const QPoint &p2)
	{
	double clockAngle = angle(p2) - angle(p1);
	return (clockAngle < 0)? clockAngle + 2*PI:clockAngle;
	}

// This is suposed to sort the points in a list, by... who knows?
// Anyway, it doesn't works
bool SortTemplatePoints(QList<QPoint> &points)
	{
	if (points.size() != 5)
		return false;

	QList<QPoint> result;

	// This array will keep the index of the points.
	uInt index[5];

	// Calculate the index of the central point
	uInt indexp = getClosestPointIndex(getMeanPoint(points), points);

	result.append(points.at(indexp));
	points.removeAt(indexp);

	// Calculate the index of the top left point
	double minDistance = 1000000;
	for (uInt n = 0; n < points.size(); n++)
		if ( points.at(n).manhattanLength() < minDistance )
			{
			minDistance = points.at(n).manhattanLength();
			indexp = n;
			}

	result.append(points.at(indexp));
	points.removeAt(indexp);

	// Calculate the other points
	while(points.size() > 0)
		{
		indexp = 0;
		double minAngle = clockWiseAngle( result.back() - result.front(), points.at(indexp) - result.front());
	
		for (uInt n = 1; n < points.size(); n++)
			{
			double actualAngle = clockWiseAngle( result.back() - result.front(), points.at(n) - result.front());
			if ( actualAngle < minAngle )
				{
				minAngle = actualAngle;
				indexp = n;
				}
			}

		result.append(points.at(indexp));
		points.removeAt(indexp);
		}

	points = result;

	return true;
	}

// This function returns the homography that maps the points from a source position to a
// destination position
Matrix<> CalibrateHomography(const Matrix<> &sourcePointsMatrix, const Matrix<> &destinationPoints)
	{
	Q_ASSERT(sourcePointsMatrix.num_cols() == 2);
	Q_ASSERT(sourcePointsMatrix.num_cols() == destinationPoints.num_cols());
	Q_ASSERT(sourcePointsMatrix.num_rows() == destinationPoints.num_rows());

	const uInt rows = sourcePointsMatrix.num_rows();

	// First, state the coefs matrix for the homogeneous linear system of equations, of the form:
	//	Ax = 0
	Matrix<> coefsMatrix(3*rows,9);

	for (uInt n = 0; n < rows; n++)
		{
		double	x = sourcePointsMatrix[n][0], y = sourcePointsMatrix[n][1],
			p = destinationPoints[n][0], q = destinationPoints[n][1];

		double	equation1[9] = {	0,	0,	0,	-x,	-y,	-1,	q*x,	q*y,	q},
			equation2[9] = {	x,	y,	1,	0,	0,	0,	-p*x,	-p*y,	-p},
			equation3[9] = {	-q*x,	-q*y,	-q,	p*x,	p*y,	p,	0,	0,	0};

		coefsMatrix[3*n] = Vector<9>(equation1);
		coefsMatrix[3*n+1] = Vector<9>(equation2);
		coefsMatrix[3*n+2] = Vector<9>(equation3);
		}

	// create the SVD of M
	SVD<> svdCoefsMatrix(coefsMatrix);

	Vector<9> x =  svdCoefsMatrix.get_VT()[8];
	Matrix<> homography(3,3);

	homography[0][0] = x[0];	homography[0][1] = x[1];	homography[0][2] = x[2];
	homography[1][0] = x[3];	homography[1][1] = x[4];	homography[1][2] = x[5];
	homography[2][0] = x[6];	homography[2][1] = x[7];	homography[2][2] = x[8];

	return homography;
	}

void normalizeHomogeneousCoordinates(Matrix<> &points)
	{
	const uInt cols = points.num_cols(), rows = points.num_rows();

	for (uInt i = 0; i < rows; i++)
		for (uInt j = 0; j < cols; j++)
			points[i][j] /= points[i][cols-1];
	}

void normalizeHomogeneousCoordinates(Matrix<1,3> &points)
	{
	const uInt cols = points.num_cols(), rows = points.num_rows();

	for (uInt i = 0; i < rows; i++)
		for (uInt j = 0; j < cols; j++)
			points[i][j] /= points[i][cols-1];
	}

// This function retunrs an integer value that indicates how well the homography maps the source points
// in a list, to the points in the destination list.
double	testErrorHomography(const Matrix<> &sourcePointsMatrix, const Matrix<> &destinationPoints, const Matrix<> homography)
	{
	const uInt cols = sourcePointsMatrix.num_cols(), rows = sourcePointsMatrix.num_rows();

	Matrix <> projectedPoints(sourcePointsMatrix.num_rows(),3);
	Matrix <> residuals (sourcePointsMatrix.num_rows(),3);

	projectedPoints = sourcePointsMatrix * homography.T();
	normalizeHomogeneousCoordinates(projectedPoints);
	residuals = projectedPoints - destinationPoints;

	double accum = 0;
	for (uInt i = 0; i < rows; i++)
		{
		double square = 0;
		for (uInt j = 0; j < cols; j++)
			square += residuals[i][j]*residuals[i][j];
		accum += sqrt(square);
		}

	return accum;
	}

// This function creates a list of the template points, first with the center point,
// then the rest of the points following a clockwise order around the center point.
Matrix<> GetTemplateMatrixPoints()
	{
	//					Center		Norwest		Noreast		Southeast	Southwest
	return Matrix<5,3> ((double[5][3]){	0,0,1,		-1,+1,+1,	+1,+1,+1,	+1,-1,+1,	-1,-1,+1});
	}

// This function sustitudes warpPerspective from the qvipp, and applies the planar transformation H
// to a source image, storing the transformed image in the destination image
void	myWarpPerspective(const QVImage<uChar> &src, QVImage<uChar> &dest, const Matrix <> H, const double zoom)
	{
	const double cols = src.getCols(), rows = src.getRows();
	const Matrix<> Hinv = SVD<>(H).get_pinv();

	for (double col = 0; col < cols; col++)
		for (double row = 0; row < rows; row++)
			{
			const double x = col, y = row;

			double	v[3] = { 2*(x - cols/2)/cols, -2*(y - rows/2)/cols, 1 };

			Vector <3> vP;
			vP = H * Vector<3>(v);
			const double x0 = vP[0]/vP[2], y0 = vP[1]/vP[2];
			const QPoint p2 =  QPoint( (uInt) x, (uInt) y ), p1 = QPoint( zoom*x0 + cols/2, -zoom*y0 + rows/2 );

			if (dest.getROI().contains(p2) && src.getROI().contains(p1))
				dest(p1) = src(p2);
			}
	}

// This function transforms a list of n points, to a nx3 matrix, containing the rows the
// homogeneous coordinates for each point in the list.
void pointListToMatrix(const QVImage<uChar> &image, const QList<QPoint> &points, Matrix<> &matrix)
	{
	Q_ASSERT(points.size() == matrix.get_rows());
	Q_ASSERT(3 == matrix.get_cols());
	const double	rows = image.getRows(), cols = image.getCols();

	for (uInt n = 0; n < points.size(); n++)
		{
		const double p = points.at(n).x(), q = points.at(n).y();
		double	v[3] = { 2*(p - cols/2)/cols, -2*(q - rows/2)/cols, 1 };
		matrix[n] = Vector<3>(v);
		}
	}

class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name): QVWorker(name)
			{
			addProperty<double>("Max error", inputFlag, 0.02, "for an homography to be considered good", 0, 0.1);
			addProperty<double>("Zoom", inputFlag, 50, "Size of the rectified template", 1, 100);
			addProperty<double>("Focal", outputFlag, 0, "Focal distance");
			addProperty<int>("Window size", inputFlag, 10, "Corner response window search", 1, 100);
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,3> >("Corners", outputFlag);
			addProperty< QVImage<uChar,1> >("Wrapped", outputFlag);
			}

		void iterate()
			{
			const QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");
			const uInt	rows = image.getRows(), cols = image.getCols(),
					sizeMax = getPropertyValue<int>("Window size");

			const double	maxError = getPropertyValue<double>("Max error"),
					zoom = getPropertyValue<double>("Zoom"),
					focal = getPropertyValue<double>("Focal");

			QVImage<uChar,3> destino = image;

			timeFlag("grab Frame");

			////////////////////////////////////
			// Harris corner response image
			QVImage<sFloat>	temp(cols, rows), cornerResponseImage(cols, rows);
			SobelCornerResponseImage(image, cornerResponseImage);

			timeFlag("Corner response image");

			////////////////////////////////////
			// Hot points
			QList<QPoint> hotPoints;
			GetHotPoints(cornerResponseImage, hotPoints, sizeMax);

			timeFlag("Get hotpoints");

			///////////////////////////////////////////
			// Calibración
			QList<QPoint> maximalPoints;
			GetMaximalPoints(cornerResponseImage, hotPoints, maximalPoints, 5);
			SortTemplatePoints(maximalPoints);

			drawPoints(maximalPoints, destino);

			timeFlag("Get max hotpoints");

			if (maximalPoints.size() == 5)
				{
				Matrix <> sourcePointsMatrix(5,3), destinationPointsMatrix(5,3);
				destinationPointsMatrix =  GetTemplateMatrixPoints();
				pointListToMatrix(image, maximalPoints, sourcePointsMatrix);

				Matrix<> H = CalibrateHomography(sourcePointsMatrix, destinationPointsMatrix);

				const double actualError = testErrorHomography(sourcePointsMatrix, destinationPointsMatrix, H);
				if (actualError < maxError)
					{
					QVImage<uChar> wrapped(cols, rows);
					Set(wrapped,0);
	
					myWarpPerspective(image, wrapped, H, zoom);

					setPropertyValue< QVImage<uChar,1> >("Wrapped", wrapped);
					}
				}

			timeFlag("Calibrate");

			setPropertyValue< QVImage<uChar,3> >("Corners", destino);
			timeFlag("Draw corners");
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		
		"Example program for QVision library. Applies corner detection over an input video."

		);

	QVMPlayerCamera camera("Video");
	MyWorker worker("Corners Worker");
	camera.link(&worker,"Input image");

	QVGUI interface;

	QVImageCanvas imageCanvas("Corners");
	imageCanvas.linkProperty(worker, "Corners");

	QVImageCanvas imageCanvas2("Wrapped");
	imageCanvas2.linkProperty(worker, "Wrapped");

	return app.exec();
	}

/// \endcond

