/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qtensorindex.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVTENSORINDEX_H
#define QVTENSORINDEX_H

#include <QList>
#include <QMap>
#include <QVector>

#define	ABS(X)		(((X)>0)?(X):(-(X)))

#ifndef DOXYGEN_IGNORE_THIS
class QTensorIndexRange;
class QTensorIndexValues: public QList<QTensorIndexRange>
	{
	public:
		QTensorIndexValues(): QList<QTensorIndexRange>()	{}

		const QTensorIndexValues operator&(const QTensorIndexRange &indexRange) const
			{
			QTensorIndexValues result = *this;
			result.append(indexRange);
			return result;
			}

		bool operator==(const QTensorIndexValues &indexList) const
			{ return true; }

		QMap<int, QVector<int> > getIndexesValues() const;
	};

class QTensorIndexRange
	{
	friend class QTensorIndex;

	public:
		int id, rangeMin, rangeMax;

		QTensorIndexRange(const QTensorIndexRange &tensorIndexRange):
			id(tensorIndexRange.id), rangeMin(tensorIndexRange.rangeMin), rangeMax(tensorIndexRange.rangeMax) { }

		const QTensorIndexValues operator&(const QTensorIndexRange &indexRange) const
			{
			QTensorIndexValues result;
			result.append(*this);
			result.append(indexRange);
			return result;
			}

		operator QTensorIndexValues() const
			{
			QTensorIndexValues result;
			result.append(*this);
			return result;
			}
	private:
		QTensorIndexRange(const int id, const int rangeMin, const int rangeMax):	id(id), rangeMin(rangeMin), rangeMax(rangeMax)	{ }
	};

//////////////////////////////////////////////
// NOTA: identificador = -1, o 1 indica que el índice no existe, no concuerda con ningún otro.
class QTensorIndex;
class QTensorValence: public QList<QTensorIndex>
	{
	public:
		QTensorValence(): QList<QTensorIndex>()	{}

		const QTensorValence operator*(const QTensorIndex &index) const
			{
			QTensorValence result = *this;
			result.append(index);
			return result;
			}

		const QTensorValence operator&(const QTensorIndex &index) const
			{
			QTensorValence result = *this;
			result.append(index);
			return result;
			}

		bool operator==(const QTensorValence &indexList) const
			{
			return true;
			}

		const QTensorValence operator+(const QTensorValence &indexList) const
			{ return *this + indexList; }

		//QMap<int, QVector<int> > getIndexesValues() const;
		QMap<int, QVector<int> > getIndexesPositions() const;
	};

#endif

/*!
@class QTensorIndex qtensorindex.h qtensor/qtensorindex.h
@brief Indexes for tensors.

The valence of a particular tensor is the number and type of array indices.
QTensorIndex class can be used to create index objects, and compose list of them to indicate the valence of a tensor, at construction.

See @ref QTensor class for detailed usage.

@see QTensor
@ingroup qtensor
*/
class QTensorIndex
	{
	public:
		/// @brief Creates a new index name object.
		///
		/// 
		QTensorIndex(const int dimension): dim(dimension), id(nextIndexId++)							{ }

		/// @brief Copy constructor for tensor index name objects.
		///
		/// Copies an index.
		QTensorIndex(const QTensorIndex &tensorIndex): dim(tensorIndex.dim), id(tensorIndex.id)					{ }

		/// @brief Gets the covariant version of this index name
		/// @return a QTensorIndex object, which is the covariant version of the original one
		QTensorIndex cov()		const { return QTensorIndex(*this, true); }

		/// @brief Gets the covariant version of this index name
		/// @return a QTensorIndex object, which is the covariant version of the original one
		QTensorIndex covariant()	const { return QTensorIndex(*this, true); }

		/// @brief Gets a value state containing a range of integer values for the index name
		/// @return an object containing a range of values for the index name
		QTensorIndexRange range(const int min, const int max)	const { return QTensorIndexRange(ABS(id), min, max); }

		/// @brief Gets a value state containing an integer, for the index name
		/// @return an object containing the value for the index name
		const QTensorIndexRange operator[](const int value)	const { return QTensorIndexRange(ABS(id), value, value); }

		#ifndef DOXYGEN_IGNORE_THIS
		QTensorIndex(): dim(0), id(1)												{ }
		QTensorIndex(const int dimension, const int ident): dim(dimension), id(ident)						{ }
		QTensorIndex(const QTensorIndex &tensorIndex, bool variate): dim(tensorIndex.dim), id((variate?-1:1)*tensorIndex.id)	{ }

		int dim, id;
		const QTensorValence operator*(const QTensorIndex &index) const
			{
			QTensorValence result;
			result.append(*this);
			result.append(index);
			return result;
			}

		bool operator==(const QTensorIndex &index) const
			{ return true; }

		operator QTensorValence() const
			{
			QTensorValence result;
			result.append(*this);
			return result;
			}

		const QTensorValence operator*(const QTensorValence &indexList) const
			{
			QTensorValence result = *this;
			result << indexList;
			return result;
			}
		#endif
	private:
		static int nextIndexId;
		static QTensorIndex anyIndex;
	};
#endif
