/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvimage.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVIMAGE_H
#define QVIMAGE_H

#include <QMetaType>
#include <qvcore/qvimagebuffer.h>

/// @brief Initializes image to be read with macro QVIMAGE_PIXEL.
///
/// This macro defines variables that will be used with macro @ref QVIMAGE_PIXEL, to read pixels in a given image.
/// An example of use of this macro can be found in section @ref Macros.
/// @param TYPE Type of the image. It should be the value of first template parameter specified for the image.
/// @param IMAGE Image name.
/// @see QVIMAGE_PIXEL
#define	QVIMAGE_INIT_READ(TYPE, IMAGE)						\
	const TYPE * __qv_data_##IMAGE##__ = IMAGE.getReadData();		\
	const uInt __qv_step_##IMAGE##__ = IMAGE.getStep()/sizeof(TYPE);	\
	const uChar __qv_planes_##IMAGE##__  = IMAGE.getChannels();

/// @brief Initializes image to be write or read with macro QVIMAGE_PIXEL.
///
/// This macro defines variables that will be used with macro @ref QVIMAGE_PIXEL, to access pixels in a given image.
/// An example of use of this macro can be found in section @ref Macros.
/// @param TYPE Type of the image. It should be the value of first template parameter specified for the image.
/// @param IMAGE Image name.
/// @see QVIMAGE_PIXEL
#define	QVIMAGE_INIT_WRITE(TYPE, IMAGE)						\
	TYPE * __qv_data_##IMAGE##__ = IMAGE.getWriteData();			\
	const uInt __qv_step_##IMAGE##__ = IMAGE.getStep()/sizeof(TYPE);	\
	const uChar __qv_planes_##IMAGE##__  = IMAGE.getChannels();

/// @brief Initializes pointed image to be read with macro QVIMAGE_PIXEL.
///
/// This macro defines variables that will be used with macro @ref QVIMAGE_PIXEL, to read pixels in a given image.
/// An example of use of this macro can be found in section @ref Macros.
/// @param TYPE Type of the image. It should be the value of first template parameter specified for the image.
/// @param IMAGE Name of the pointer variable that points to the image.
/// @see QVIMAGE_PIXEL
#define	QVIMAGE_PTR_INIT_READ(TYPE, IMAGE)					\
	const TYPE * __qv_data_##IMAGE##__ = IMAGE->getReadData();		\
	const uInt __qv_step_##IMAGE##__ = IMAGE.getStep()/sizeof(TYPE);	\
	const uChar __qv_planes_##IMAGE##__  = IMAGE->getChannels();

/// @brief Initializes pointed image to be write or read with macro QVIMAGE_PIXEL.
///
/// This macro defines variables that will be used with macro @ref QVIMAGE_PIXEL, to access pixels in a given image.
/// An example of use of this macro can be found in section @ref Macros.
/// @param TYPE Type of the image. It should be the value of first template parameter specified for the image.
/// @param IMAGE Name of the pointer variable that points to the image.
/// @see QVIMAGE_PIXEL
#define	QVIMAGE_PTR_INIT_WRITE(TYPE, IMAGE)					\
	TYPE * __qv_data_##IMAGE##__ = IMAGE->getWriteData();			\
	const uInt __qv_step_##IMAGE##__ = IMAGE.getStep()/sizeof(TYPE);	\
	const uChar __qv_planes_##IMAGE##__  = IMAGE->getChannels();

/// @brief Access pixel from image previously initialized with macros QVIMAGE_INIT_READ, QVIMAGE_INIT_WRITE, QVIMAGE_PTR_INIT_READ, or QVIMAGE_PTR_INIT_WRITE.
///
/// This macro can be used to access arbitrary pixels of an image, with a good time performance.
/// An example of use of this macro can be found in section @ref Macros.
/// @see QVIMAGE_INIT_READ
/// @see QVIMAGE_INIT_WRITE
/// @see QVIMAGE_PTR_INIT_READ
/// @see QVIMAGE_PTR_INIT_WRITE.
#define QVIMAGE_PIXEL(IMAGE, Col, Row, Channel)	\
	(__qv_data_##IMAGE##__ [(Row)* __qv_step_##IMAGE##__ + __qv_planes_##IMAGE##__ *(Col)+(Channel)])

/*! @class QVGenericImage qvimage.h qvcore/qvimage.h
@brief Base class for @ref QVImage objects.

For instance, this class is usefull to create code like this:

\code
int imageAreaSize(QVGenericImage *image)
	{
	return image->getCols() * image->getRows();
	}
\endcode

Where it doesn't matter the bit depth or number of channels of the image, but the number of cols and rows of it, as well as generic containers, that can hold a variety of image types, for example:

\code
class GenericImageContainer
	{
	public:
		int count;
		QVGenericImage *images;
		[...]
		void addImage(QVGenericImage *image)	{ images[count++] = &image; }
		QVGenericImage getImage(int index)	{ return images[index]; }
	}
\endcode

Thus, this class offers access methods for all kind of information about the image, like dimensions (see @ref getCols() and @ref getRows() methods), bit depth and number of planes (see @ref getTypeSize() and @ref getChannels() methods), ROI and anchor of the image (see @ref getROI() and @ref getAnchor() methods mainly), but pixel accessor methods, that are defined in the subclass @ref QVImage, and depends on the bit depth and number of planes of the image.

@see QVImage
@ingroup qvcore
*/
class QVGenericImage
	{
	public:
		/// @brief Default constructor.
		///
		/// Initializes image ROI and anchor to zero offset and width, both.
		QVGenericImage (): roi(0,0,0,0), anchor(QPoint(0,0))					{ }

		/// @brief Copy constructor.
		///
		/// Initializes image ROI and anchor to ROI and anchor from image 'img'.
		QVGenericImage(QVGenericImage const &img): roi(img.getROI()), anchor(img.getAnchor())	{ }

		/// @brief Virtual destructor for QVGenericImage.
		virtual ~QVGenericImage ()								{ };

		/// @brief Virtual method for getting number of columns from image.
		///
		/// @returns number of columns for image,
		virtual uInt	getCols()		const	= 0;

		/// @brief Virtual method for getting number of rows from image.
		///
		/// @returns number of rows for image,
		virtual uInt	getRows()		const	= 0;

		/// @brief Virtual method for getting step from image.
		///
		/// This functions returns the distance in bytes between starts of consecutive lines in the image.
		///
		/// This may be not equal to a multiple of the number of columns, so this value is necessary to normal access
		/// to the image.
		///
		/// @returns distance in bytes between two consecutive lines in the image.
		virtual uInt	getStep()		const	= 0;

		/// @brief Virtual method for getting number of planes for the image.
		///
		/// Returned value should be equal to template parameter 'Channels', for any QVImage. This is the number
		/// of channels that can be accessed in the image.
		///
		/// @returns number of channels of the image.
		virtual uInt	getChannels()		const	= 0;

		/// @brief Virtual method for getting bit depth of the image, in bytes.
		///
		/// Returned value should be equal to size of template parameter 'Type', for any QVImage. This is the size
		/// of any element of the data array that stores the image in memory.
		///
		/// @returns number of channels of the image.
		virtual	uInt	getTypeSize()		const	= 0;

		/// @brief Virtual method for getting data size for the image.
		///
		/// This functions, for any image, returns the following value:
		/// \verbatim dataSize = rows * step * planes \endverbatim
		///
		/// @returns size in bytes of data buffer for the image.
		virtual	uInt	getDataSize()		const	= 0;

		/// @brief Method for obtaining actual region of interest for image.
		///
		/// The region of interest of an image is used for various operations over it. Check the manual for further
		/// documentation
		///
		/// @returns a QRect that contains the region of interest of the image.
		const QRect &	getROI()		const	{ return roi; }

		/// @brief Obtains actual anchor for image.
		///
		/// The anchor of an image is used for various operations over it. Check the manual for further
		/// documentation
		///
		/// @returns a QPoint that indicates anchor of the image.
		const QPoint &	getAnchor()		const	{ return anchor; }

		// ROI operations
		/// @brief Resets the region of interest of the image.
		///
		/// Makes the region of interest of the image hold the whole image.
		///
		/// That is, makes the region of interest be the rectangle which has upper left corner in pixel (0,0),
		/// and lower right corner in pixel (columns, rows), where columns and rows are the number of columns
		/// and rows of the image.
		///
		void resetROI()				{ setROI(0,0,getCols(), getRows()); }

		/// @brief Reduces the region of interest of the image, in a given amount of pixels.
		///
		/// Adds cols and rows parameters to the top left corner of the region of interest, and those values
		/// doubled, to the width.
		///
		/// That is, reduces the region of interest in a border, of columns and rows indicated in the parameters.
		///
		/// @param cols	columns which should be reduced from the border of the roi.
		/// @param rows	rows which should be reduced from the border of the roi.
		void erodeROI(uInt cols, uInt rows)
			{ setROI(getROI().x()+cols, getROI().y()+rows, getROI().width()-2*cols, getROI().height()-2*rows); }

		/// @brief Increments the region of interest of the image, in a given amount of pixels.
		///
		/// Substracts cols and rows parameters to the top left corner of the region of interest, and those values
		/// doubled, to the width.
		///
		/// That is, increments the region of interest in a border, of columns and rows indicated in the parameters.
		///
		/// @param cols	columns which should be incremented to the border of the roi.
		/// @param rows	rows which should be incremented to the border of the roi.
		void dilateROI(uInt cols, uInt rows)
			{ setROI(getROI().x()-cols, getROI().y()-rows, getROI().width()+2*cols, getROI().height()+2*rows); }

		/// @brief Sets dimensions and location of region of interest in an image.
		///
		/// Region of interest for an image is a rectangle significant to many operations that can be done over an
		/// image.
		///
		/// It is defined by pixel where top left corner for this rectangle is located, and width & height of
		/// this rectangle.
		///
		/// @param x column for the pixel where top left corner of the region of interest is located.
		/// @param y row for the pixel where top left corner of the region of interest is located.
		/// @param w width for the region of interest.
		/// @param h height for the region of interest.
		void setROI(int x, int y, uInt w, uInt h) { setROI(QRect(x,y,w,h)); }

		/// @brief Sets the region of interest for an image, holding the whole image, except a border of a given width.
		///
		/// @param margin size of the border.
		void setMarginROI(int margin)
			{ setROI(margin, margin, getCols() - 2*margin, getRows() -2*margin); }

		/// @brief Directly sets the region of interest for an image, with a given rectangle.
		///
		/// @param rect rectangle that represents region of interest.
		void setROI(const QRect &rect)
			{
			Q_ASSERT_X(rect.x() >= 0,"QVGenericImage::setROI()","QRect.x() is less than zero");
			Q_ASSERT_X(rect.y() >= 0,"QVGenericImage::setROI()","QRect.y() is less than zero");
			Q_ASSERT_X(rect.width() > 0,"QVGenericImage::setROI()","QRect.width() is less or equal to zero");
			Q_ASSERT_X(rect.height() > 0,"QVGenericImage::setROI()","QRect.height() is less or equal to zero");
			Q_ASSERT_X(rect.x()+rect.width() <= (int) getCols(),"QVGenericImage::setROI()","x + width > columns");
			Q_ASSERT_X(rect.x()+rect.height() <= (int) getRows(),"QVGenericImage::setROI()","y + height > rows");
			roi = rect;
			}

		// Anchor operations
		/// @brief Sets anchor to pixel (0,0).
		///
		void resetAnchor()			{ setAnchor(0,0); }

		/// @brief Sets anchor to a given pixel coordinates.
		///
		/// @param col column of the pixel where the anchor is located.
		/// @param row row of the pixel where the anchor is located.
		void setAnchor(int col, int row)	{ setAnchor(QPoint(col,row)); }

		/// @brief Directly sets anchor to a given pixel coordinates, given a point.
		///
		/// @param point QPoint where the anchor is located.
		void setAnchor(const QPoint &point)
			{
			Q_ASSERT_X(point.x()>=0,"QVGenericImage::setAnchor()","horizontal value for anchor is less than zero");
			Q_ASSERT_X(point.y()>=0,"QVGenericImage::setAnchor()","vertical value for anchor is less than zero");
			Q_ASSERT_X(point.x() < (int) getCols(),"QVGenericImage::setAnchor()","horizontal value exceeds cols");
			Q_ASSERT_X(point.y() < (int) getRows(),"QVGenericImage::setAnchor()","vertical value exceeds rows");
			anchor = point;
			}

		/// @brief Function to make introspection in the image, and obtain a char string identifying the type of the
		/// real image contained in the QVGenericImage.
		///
		/// @returns pointer to a constant char string identifying image type.
		virtual const char * getTypeQString() const = 0;

		/// @brief Function to check whether an image is compatible with given image type id char string.
		///
		/// This functions checks if this image has same bit depth and channel number, as the type of QVGenericImage
		/// represented by the parameter string.
		///
		/// Id char string can be that returned by getTypeQString() function, from another QVGenericImage, or
		/// one specified by the programmer.
		///
		/// This function is used to dynamic type introspection, to determine subclass of a QVGenericImage, and
		/// perform different functionality with it, depending on bit depth or number of planes. An example code for
		/// this use is below:
		/// \code [...]
		/// QVImage image;
		///
		/// [...]
		///
		/// if(image->isCompatibleWith("QVImage<uChar,1>"))
		///	// do stuff with the uChar bit depth, one channel image ...
		/// else if(image->isCompatibleWith("QVImage<uChar,3>"))
		///	// do stuff with the uChar bit depth, three channel image ...
		/// [...]
		/// else
		///	// alternate code for that type of image ...
		///
		/// [...] \endcode
		///
		/// @param qvimageClassName char string identifying QVGenericImage type of an image.
		/// @returns true if this image is compatible in type with that represented by the string.
		bool isCompatibleWith(const char *qvImageClassName) const
			{
			Q_ASSERT_X(qvImageClassName!= NULL,"QVGenericImage::isCompatibleWith()","class name string is NULL");
			return (0 == strcmp(this->getTypeQString(),qvImageClassName));
			}

		/// @brief Function to check whether an image is compatible with this image.
		///
		/// This functions checks if this image is compatible with a given image (that is, it has same bit depth and
		/// same number of channels).
		///
		/// Id char string can be that returned by getTypeQString() function, from another QVGenericImage, or
		/// one specified by the programmer.
		///
		/// @param image image to check type with.
		/// @returns true if this image is compatible in type with that represented by the string.
		bool isCompatibleWith(const QVGenericImage *image) const
			{
			Q_ASSERT_X(image!= NULL,"QVGenericImage::isCompatibleWith()","NULL pointer");
			return this->isCompatibleWith(image->getTypeQString());
			}

	protected:
		QRect roi;
		QPoint anchor;
	};

/*!
@class QVImage qvimage.h qvcore/qvimage.h
@brief Image representation class for the QVision.

QVImage class extends @ref QVGenericImage, and wraps image data in a simple and lightweight structure, easy to use. It is a template class, as you can see in the header of the class:

\code
template <typename Type, int Channels = 1> class QVImage: public QVGenericImage 
	{
	[...]
	}
\endcode
parametrized with the data type of the pixels in the image, and the number of channels that it contains.

\section ImageTypes Image Types
\subsection qvimagebit-depth Image bit-depth.
When creating an image, it's bit-depth is specified with the first template argument <i>Type</i>.

Virtually, you can specify pixel types of any kind, but QVision has defined functionality to process pixels of the following types:

<table>
<tr><th>C++ type</th>		<th>QVision type</th>	<th>IPP type</th>	<th>Bit depth</th></tr>
<tr><td>unsigned char</td>	<td>uChar</td>		<td>Ipp8u</td>		<td>8</td></tr>
<tr><td>unsigned short</td>	<td>uShort</td>		<td>Ipp16u</td>		<td>16</td></tr>
<tr><td>unsigned int</td>	<td>uInt</td>		<td>Ipp32u</td>		<td>32</td></tr>
<tr><td>signed char</td>	<td>sChar</td>		<td>Ipp8s</td>		<td>8</td></tr>
<tr><td>signed short</td>	<td>sShort</td>		<td>Ipp16s</td>		<td>16</td></tr>
<tr><td>signed int</td>		<td>sInt</td>		<td>Ipp32s</td>		<td>32</td></tr>
<tr><td>typedef float</td>	<td>sFloat</td>		<td>Ipp32f</td>		<td>32</td></tr>
<tr><td>typedef double</td>	<td>sDouble</td>	<td>Ipp64f</td>		<td>64</td></tr>
</table>

In practice, most of the functions will work with images of type <i>uChar</i>, <i>sShort</i>, and <i>sFloat</i>.

\subsection Channel Number of channels in an image.

Each image stores a vector of values of type indicated in the first parameter of the template for each pixel, of size indicated in the second parameter of the template. The latter is called <i>Channel number</i> or simply <i>Channels</i>, and it indicates the number of channels in which the image stores the data.

For example, the following code line creates images of 8 bit-depth, 16 bit-depth, and 32 bit-depth, of one and three channels:

\code
QVImage<uChar, 1> image8uC1;
QVImage<sShort, 1> image16sC1;
QVImage<sInt, 1> image32sC1;

QVImage<uChar, 3> image8uC3;
QVImage<sShort, 3> image16sC3;
QVImage<sInt, 3> image32sC3;
\endcode

You can ommit the number of channels, if you want to create a one channel image, because that will be the default value for that template parameter. Thus the first three code lines of the previous example can be written like this:

\code
QVImage<uChar> image8uC1;
QVImage<sShort> image16sC1;
QVImage<sInt> image32sC1;
\endcode

with identic results.

As in the case of the <i>Type</i> parameter, this parameter can be assigned any arbitrary positive integer value, but generally there wouldn't be functionallity to work with images with a channel number different of one or three channels.

\subsection Convert Conversion between image types.
When using the constructor and the assign operator with a source image different in type or channel number to the created or destination image respectively, QVision authomatically performs a conversion. Conversion generally calls to the proper <i>ippConvert</i> function, so a review to @ref Convert function is recommended for futher learning about this feature of QVImage objects.

\warning It is not possible to directly convert or construct an image from another one that differs in both type and in channel number. It should be converted to an intermediate image that shares the same number of channels and bit-depth than both images first.

\section PixelAccess Pixel access.
Access to pixel in the data buffer can be done in three different ways: using operator (), direct access to the data buffer, and using QVIMAGE_* macros.

\subsection OperatorParentesis Using operator ()
 Operator () is overloaded in class QVImage to read pixel values, both to read and write pixels in the image. This is the least efficient access method in execution time performance, but the easiest one to use, and the safest. It performs some sanity checks, to avoid reading from a pixel outside the image (thus, also outside the image data buffer). An example of use follows:

\code
QVImage<uChar> imageCharBis = imageChar;
[...]
// This line assigns the value 0 to pixel (10,10) in the image imageChar:
imageCharBis(10,10) = 0;
// And this line reads value of pixel (11,11) of the image imageChar in the variable value:
uChar value = imageCharBis(11,11);

[...]

// The following code shows how to read and write all the pixels from QVImage's of size 100x100:
QVImage<uChar> imageChar1(100,100), imageChar2(100,100);
[...]
for (uInt row = 0; row < imageChar1.getRows(); row++)
	for (uInt col = 0; col < imageChar1.getCols(); col++)
		imageCharData2(col, row) = imageCharData1(col, row);
[...]
\endcode

\subsection DataBuffer Direct access through data buffer.
The data buffer of an image can be accessed directly with methods operator() getReadData() and getWriteData(), for reading or reading/writting access respectively. This way of accessing image pixels is the fastest, and the ugliest, also has some problems that will be noticed latter.

To access any pixel getStep(), getRows() and getCols() methods should be used, to get the width in bytes of the lines, the number of lines, and the number of valid elements in each line of the image data buffer, respectively. Follows an example of directly accessing the data buffer of an image using these methods:

\code
QVImage<uChar> imageChar1(100,100), imageChar2(100, 100);
[...]
uInt		step1 = imageChar1.getStep(), step2 = imageChar2.getStep();
uChar		*imageCharData1 = imageChar1.getReadData(),
	const	*imageCharData2 = imageChar2.getWriteData();

for (uInt row = 0; row < imageChar1.getRows(); row++)
	for (uInt col = 0; col < imageChar1.getCols(); col++)
		imageCharData2[row*step2 + col] = imageCharData1[row*step1 + col];
[...]
\endcode

\warning If you keep a pointer to the buffer data of an image after using a method or function that modifes it, it can become obsolete, due to copy-on-write. EXPLICAR MEJOR ESTO.

\subsection Macros Using macros QVIMAGE_INIT_*, and QVIMAGE_PIXEL.
This is the last way of pixel reading/writting on an image, and is a compromise between code simplicity and execution time performance.

Macro @ref QVIMAGE_PIXEL can be used to access a pixel from an image previously initialized to be used with that macro, with macros @ref QVIMAGE_INIT_READ and @ref QVIMAGE_INIT_WRITE. An example of this follows:

\code
QVImage<sFloat, 1> imageFloat(100,100);
QVImage<uChar, 3> imageChar3(100,100);
[...]
QVIMAGE_INIT_READ(sFloat,imageFloat);
QVIMAGE_INIT_WRITE(uChar,imageChar3);
for(uInt row = 0; row < imageFloat.getRows(); row++)
	for(uInt col = 0; col < imageFloat.getCols(); col++)
		QVIMAGE_PIXEL(imageChar3, col, row,1) = QVIMAGE_PIXEL(imageFloat, col, row,0);
[...]
\endcode

Macro QVIMAGE_PIXEL always takes 4 parameters: image name, row, column, and channel number, starting with number 0 for the first channel. In one channel images, only 0 value should be  valid for this last parameter.

Macros @ref QVIMAGE_PTR_INIT_READ and @ref QVIMAGE_PTR_INIT_WRITE work like macros @ref QVIMAGE_INIT_READ and @ref QVIMAGE_INIT_WRITE, only that they are given a pointer to the image to be accessed with macro @ref QVIMAGE_PIXEL, not the name of the image itselft. An example of use of these macros follows:

\code
QVImage<sFloat, 1> * ptrImageFloat(100,100);
QVImage<uChar, 3> * ptrImageChar3(100,100);
[...]
QVIMAGE_PTR_INIT_READ(sFloat,ptrImageFloat);
QVIMAGE_PTR_INIT_WRITE(uChar,ptrImageChar3);
for(uInt row = 0; row < ptrImageFloat.getRows(); row++)
	for(uInt col = 0; col < ptrImageFloat.getCols(); col++)
		QVIMAGE_PIXEL(ptrImageChar3, col, row,1) = QVIMAGE_PIXEL(ptrImageFloat, col, row,0);
[...]
\endcode

Use of macro @ref QVIMAGE_PIXEL is the same for all of the initialization macros.

\warning As with the direct access method, this one can save an old copy to the pointer of the image data buffer, if the image is accessed in write mode after being initialized.

\section ROIandAnchor Regions of interest (or ROI's) and anchors
These are two properties defined in the class @ref QVGenericImage, so they are inherited by class @ref QVImage. They are very relevant for image processing functionality, in a way that restrict the area of the input image, to a rectangular area inside the image, and the destination location of the updated area in the resulting image, for an image processing method or function.

When using functions that modify the content of an image, sometimes they don't change the whole content of the image, but a small area inside it. Generally, that fraction has rectangular shape, and is called <i>region of interest</i>, or ROI. For instance that happens with some functions from the package @ref qvipp, like @ref FilterGauss, in which the area modified in the resulting image is smaller in size than the area processed in the input image, by a margin of size the width of the convolution filter matrix used.

Also, it could be convenient that only the modifed area were considered by following image functions when using that filtered image as an input image. That's why the information about the ROI is stored in the image, from one operation to another, in a way that is transparent to the programmer.

Originally, every image is created with a ROI that contains the whole image. In latter operations, that ROI may change in size, normally decreasing.

When the destination ROI of the operation over an image is smaller than the image, it should be specified were to locate the resulting area in the image. Generally, the resulting ROI of any operation will be placed in the top left corner of the image, but in some cases can be interest to locate it elsewere in the image. That position can be specified previously to operations with the anchor property.

\subsection UsageROI ROI usage
The ROI of an image is stored in the class @ref QVGenericImage as a QRect object. All of the functions in the @ref qvipp package, and many functions from the @ref qvdta package take use of the ROI value in their operations. That means they only process the area of the source images contained in the ROI. An example code for this follows:

\code
[...]

QVImage<uChar> sourceImage = tajMajal;
sourceimage.setROI(10,10,90,90);	// This specifies an anchor vector for destination image.

QVImage<uChar> destinationImage(200, 200);
Copy(sourceimage, destinationImage);

[...]
\endcode

Figure below shows the result of aplying that code to a given picture. On the left is the source image, with the specified ROI depicted as a hollow rectangle. On the right is the destination image as it is left by the code. We see that the content of the ROI area has been copied to the top left corner of the destination image:

\image html tajmajalcopyroi.png

When using functions that operate over an input image and store the result of their operations on an output image of bigger size, it could be convenient to indicate that not all of the pixels in the output image contain values derived from the input image. In other cases for some image functions, mainly filtering functions such as @ref FilterGauss, @ref FilterMedian, even if both input and output images have the same size not all of the pixels of the resulting image have values derived from the input image.

It is denoted by <i>valid data</i> to the set of pixels in the image returned or modified by a function, that depend on the input image (or images) of the function, or were modified by that function, and are considered to contain relevant data.

The main function of the ROI is not restricting manually the working area for image functions, but storing the area that contains valid data in the return image. Any function that returns an image should store in its ROI the rectangular area of valid data that it wrote on the image.

An example of why is usefull to keep the ROI in the resulting images after operations follows:

\code
[...]

QVImage<uChar> sourceImage = lena;
sourceImage.setROI(100, 110, 90, 40);	// This specifies an initial ROI for source image.

QVImage<uChar> temporaryImage(256, 256);
FilterGauss(sourceImage, temporaryImage, 5);

QVImage<uChar> destinationImage(256, 256);
FilterGauss(temporaryImage, destinationImage, 5);

[...]
\endcode

You can see the results of aplying that code to an input image in the following figure:

\image html lenagauss_5_5roi.png

On the left is the source image, with the starting ROI depicted as a hollow rectangle. On the right is the destination image as it is left by the code. In the middle is depicted the temporary image. We see that the function @ref FilterGauss modifes an area of the resulting image that is 4 pixels smaller in width and height than the input image, if used a mask of size 5.

Storing the ROI between calls to that function, in the intermediate images, allows to aboid specifying the valid area size and location in the image for each call to the filtering function.

\subsection Anchor Anchor usage
The anchor is a vector of two coordinates, stored in the class @ref QVGenericImage as a QPoint value. Functions that make use of ROI and anchor of the images should write their result in the resulting image starting at the pixel indicated by the anchor. You can see that in the following code:

\code
[...]

QVImage<uChar> sourceImage = tajMajal;
sourceImage.setROI(10,10,90,90);		// This specifies an initial ROI for source image.

QVImage<uChar> destinationImage(200, 200);
destinationImage.setAnchor(60,30);		// This specifies an anchor vector for destination image.

Copy(sourceImage, destinationImage);

[...]
\endcode

Result of using that code with <i>Taj Majal</i> image from the previous examples is depicted in the following figure:

\image html tajmajalcopyroianchor.png

You can see that the output ROI of the destination image has the top left corner in the pixel pointed by the anchor. Not like the example in section @ref UsageROI, where the destination ROI started at the top left corner of the image. You can see that you should specify the anchor <b>in the output image</b> for the image function to make use of it.

\subsection ROIandAnchorInSeveralImages Combining use of ROI and Anchor in several images
When using multiple image input functions, generally the ROI of all of them should have the same size, and may be located anywhere in the image. Following code will show it:

\code
[...]

QVImage<uChar>	sourceImage1 = lena,		// lena is an image of size 256x256
		sourceImage2 = tajMajal;	// tajMajal is an image of size 200x200
sourceImage1.setROI(45,65,155,85);		// This specifies an initial ROI for image sourceImage1.
sourceImage2.setROI(30,20,155,85);		// This specifies an initial ROI for image sourceImage2.

QVImage<uChar> destinationImage(230,230);	// destinationImage is an image of size 230x230
destinationImage.setAnchor(10,20);		// This specifies an anchor vector for image destinationImage.

Add(sourceImage1, sourceImage2, destinationImage);

[...]
\endcode

This figure shows the result of using both Lena and Taj Majal images with that code:

\image html tajmajallenacopyroianchor.png

In this case function @ref Add gets the contents of the ROI's of both input images and writes in the output image, starting at the pixel pointed by the anchor of the output image, the mean of every two pixels in the ROI's of the input images (add function uses by default a factor of 0.5 for the sum of the pixels).

\warning As it is said, ROI and anchor properties mainly affect functions in the packages @ref qvipp and @ref qvdta, and not to all of them. They do not affect pixel accessing methods described in previous section @ref PixelAccess, nor image display using @ref QVImageCanvas viewer.

@ingroup qvcore

*/

template <typename Type, int Channels = 1> class QVImage: public QVGenericImage 
	{
	public:
		/// @brief Default constructor.
		///
		/// Initializes image with a data buffer of size 1x1. Sets anchor to pixel (0,0), and region of interest
		/// to a rectangle containing the only pixel of the image.
		///
		QVImage():QVGenericImage()
			{
			this->imageBuffer = new QVImageBuffer<Type, Channels>(1, 1);
			setROI(0,0, this->imageBuffer->getCols(), this->imageBuffer->getRows());
			setAnchor(0,0);
			}

		/*! @brief Dimensions and buffer constructor.
		
		\deprecated You shouldn't be able to specify a pointer to a buffer. If you need a image data buffer, create an image, and use it's.
		Creates an image of dimension cols x rows, and optionally you can define a given step for the image.
		If buffer points to a region of memory, it will use that region as image data. Step paramameter should be the width of the lines in
		the data buffer, if the parameter @ref buffer points to a data buffer in memory.
		
		An example of this can be seen in the following code:
		
		\code
		uChar data_buffer[10][12] =	{
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 },
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 },
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 },
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 },
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 },
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 },
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 },
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 },
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 },
						{ 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 3, 7 }
						};
		
		// Creates a QVImage of size 10x10, ROI covering the whole image, and anchor pointing at pixel (0,0).
		// Data buffer of the image will point to matrix data_buffer.
		
		// The final values of each line, those in 11th and 12th columns will not be considered as part of the
		// image. Step is the length of the lines of data_buffer, 12, not the number of columns.
		QVImage<uChar> imageCharBuffer(10,10, 12, data_buffer);
		\endcode
		
		@param cols number of columns for the image.
		@param rows number of rows for the image.
		@param step defined step for the data buffer of the image. It should always be greater than the number of columns.
		@param buffer pointer to a memory zone where image is and will be stored.
		*/
		QVImage(uInt cols, uInt rows, uInt step = 0, const Type * buffer = NULL):QVGenericImage()
			{
			this->imageBuffer = new QVImageBuffer<Type, Channels>(cols, rows, step, buffer);
			setROI(0,0, cols, rows);
			setAnchor(0,0);
			}

		/// @brief Copy constructor
		///
		/// This constructor can be used to construct an image from another one with same bit depth and channel number,
		/// or from an image differing in type or number of channels.
		///
		/// In the former case, this constructor will do a copy-on-write, duplicating the pointer to the data buffer of
		/// the image, and efectively copying the data only when modifying it if shared by more than one QVImage
		/// object.
		///
		/// In the latter case, it will always allocate a new data buffer for the image, and make a conversion from
		/// different bit depth or channel number. Note that you can never convert from an image that differs in
		/// data type and bit-depth with the constructed image.
		///
		/// This constructor will copy the ROI and anchor of the parameter image.
		///
		/// @param img image to be copied.
		QVImage(QVImage<uChar,1> const &img);

		/// @brief Overloaded copy constructor.
		QVImage(QVImage<uChar,3> const &img);

		/// @brief Overloaded copy constructor.
		QVImage(QVImage<sShort,1> const &img);

		/// @brief Overloaded copy constructor.
		QVImage(QVImage<sShort,3> const &img);

		/// @brief Overloaded copy constructor.
		QVImage(QVImage<sFloat,1> const &img);

		/// @brief Overloaded copy constructor.
		QVImage(QVImage<sFloat,3> const &img);
		
		/// @brief Overloaded function from QVGenericImage::getTypeQString()
		const char * getTypeQString() const;

		/// @brief Overloaded function from QVGenericImage::getRows()
		uInt getRows()		const	{ return imageBuffer->getRows(); }

		/// @brief Overloaded function from QVGenericImage::getCols()
		uInt getCols()		const	{ return imageBuffer->getCols(); }

		/// @brief Overloaded function from QVGenericImage::getStep()
		inline uInt getStep()	const	{ return imageBuffer->getStep(); }

		/// @brief Overloaded function from QVGenericImage::getChannels()
		inline uInt getChannels()	const	{ return imageBuffer->getChannels(); }

		/// @brief Overloaded function from QVGenericImage::getDataSize()
		uInt getDataSize()	const	{ return imageBuffer->getDataSize(); }

		/// @brief Overloaded function from QVGenericImage::getTypeSize()
		uInt getTypeSize()	const	{ return imageBuffer->getTypeSize(); }

		/// @brief Method to obtain image data buffer, in read mode
		///
		/// This method can be used to program fast access algorithms to pixel values in an image.
		///
		/// It is intended to be used only to read pixels in an image, thus getWriteData() method should be used
		/// when access to write or read/write is intended to be done.
		///
		/// This function will generally be faster than getWriteData(), because it will not need to realize copy on write,
		/// in the case that the image data buffer where shared with other QVImage.
		///
		const Type * getReadData()	const	{ return imageBuffer->getReadData(); }

		/// @brief Method to obtain image data buffer, in read/write mode
		///
		/// This method can be used to program fast access algorithms to pixel values in an image.
		///
		/// This function will perform copy on write in the case that the data buffer for this image where shared with
		/// other QVImage's.
		///
		/// Thus can be slower than getReadData() method to access image pixels, but will ensure avoiding side
		/// effects on modifying shared buffers with other images.
		///
		Type * getWriteData()		{ return imageBuffer->getWriteData(); }

		/// @brief Sets pixel values for an image, to a given value.
		///
		/// This method  uses the region of interest of the image, to set pixels inside it to a given value.
		/// It can be used either for one channel, or several channel images.
		///
		/// In the former case, it should be used with one parameter, only specifying value for the first channel,
		/// otherwise, values for other channels will be ignored.
		/// If no parameters are given, the image region of interest will be set to zero value pixels.
		///
		/// In the latter case, default values of 0 will be used if there are unspecified channel values.
		///
		/// @param c1 value to set pixel's first channel in the region of interest of the image.
		/// @param c2 value to set pixel's second channel in the region of interest of the image.
		/// @param c3 value to set pixel's third channel in the region of interest of the image.
		void set(Type c1 = 0, Type c2 = 0, Type c3 = 0);

		/// @brief Operator to access pixel values for a pixel channel, given it's coordinates.
		///
		/// This operator can be used either with one channel or multichannel images.
		/// In the first case, it shouldn't be given a channel number, to use the default zero value.
		///
		/// @param col column number of pixel to be accessed.
		/// @param row row number of pixel to be accessed.
		/// @param channel channel number of pixel to be accessed.
		inline Type &operator()(const uInt col, const uInt row, const uInt channel = 0)
			{
			Type * data = imageBuffer->getWriteData();
			uInt step = getStep();

			Q_ASSERT_X(col < getCols(),"QVImage::operator()","col further upper bound");
			Q_ASSERT_X(row < getRows(),"QVImage::operator()","row further upper bound");
			int idx = row*step/sizeof(Type) + Channels*col+channel;
			Q_ASSERT_X(idx >= 0,"QVImage::operator()","accessing below data");
			Q_ASSERT_X(idx < imageBuffer->getDataSize(),"QVImage::operator()","accessing above data");
			return data[idx];
			}

		inline Type operator()(const uInt col, const uInt row, const uInt channel = 0) const
			{
			Type const * data = imageBuffer->getReadData();
			uInt cols = getCols(), rows = getRows();
			uInt step = getStep();
			int dataSize = getDataSize();

			Q_ASSERT_X(col < cols,"QVImage::operator()","row out of upper bound");
			Q_ASSERT_X(row < rows,"QVImage::operator()","row out of upper bound");
			int idx = row*step/sizeof(Type) + Channels*col+channel;
			Q_ASSERT_X(idx >= 0,"QVImage::operator()","accessing below data");
			Q_ASSERT_X(idx < dataSize,"QVImage::operator()","accessing above data");
			return data[idx];
			}

		/// @brief Overload of operator (), to access pixel values for a pixel channel, given a QVPoint and a
		/// channel number.
		///
		/// This overload is like the previous, but can be used with a QPoint, to access a concrete pixel in the image.
		/// @param point QPoint indicating location of the pixel to be accessed.
		/// @param channel channel number of pixel to be accessed.
		inline Type &operator()(const QPoint point, const uInt channel = 0)
			{
			Type * data = imageBuffer->getWriteData();
			uInt cols = getCols(), rows = getRows(), step = getStep();
			int dataSize = getDataSize();

			Q_ASSERT_X(point.x() < (int) cols,"QVImage::operator()","col further upper bound");
			Q_ASSERT_X(point.y() < (int) rows,"QVImage::operator()","row further upper bound");
			int idx = point.y()*step/sizeof(Type) + Channels*point.x()+channel;
			Q_ASSERT_X(idx >= 0,"QVImage::operator()","accessing below data");
			Q_ASSERT_X(idx < dataSize,"QVImage::operator()","accessing above data");
			return data[idx];
			}

		inline Type operator()(const QPoint point, const uInt channel = 0) const
			{
			Type const * data = imageBuffer->getReadData();
			uInt cols = getCols(), rows = getRows(), step = getStep();
			int dataSize = getDataSize();

			Q_ASSERT_X(point.x() < (int) cols,"QVImage::operator()","row out of upper bound");
			Q_ASSERT_X(point.y() < (int) rows,"QVImage::operator()","row out of upper bound");
			int idx = point.y()*step/sizeof(Type) + Channels*point.x()+channel;
			Q_ASSERT_X(idx >= 0,"QVImage::operator()","accessing below data");
			Q_ASSERT_X(idx < dataSize,"QVImage::operator()","accessing above data");
			return data[idx];
			}

		/// @brief Copy operator
		///
		/// Like the copy constructor, this operator can be used to copy an image from another one with same bit depth
		/// and channel number, or from an image differing in type or number of channels.
		///
		/// In the former case, this operator will do a copy-on-write, duplicating the pointer to the data buffer of
		/// the image, and efectively copying the data only when modifying it if shared by more than one QVImage
		/// object.
		///
		/// In the latter case, it will always allocate a new data buffer for the image, and make a conversion from
		/// different bit depth or channel number. Note that you can never convert from an image that differs in
		/// data type and bit-depth with the constructed image.
		///
		/// The copy operator will copy the ROI and anchor of the parameter image.
		///
		/// @param img image to be copied.
		QVImage<Type, Channels> & operator=(const QVImage<uChar, 1> &sourceImage);

		/// @brief Overloaded copy operator.
		QVImage<Type, Channels> & operator=(const QVImage<uChar, 3> &sourceImage);

		/// @brief Overloaded copy operator.
		QVImage<Type, Channels> & operator=(const QVImage<sShort, 1> &sourceImage);

		/// @brief Overloaded copy operator.
		QVImage<Type, Channels> & operator=(const QVImage<sShort, 3> &sourceImage);

		/// @brief Overloaded copy operator.
		QVImage<Type, Channels> & operator=(const QVImage<sFloat, 1> &sourceImage);

		/// @brief Overloaded copy operator.
		QVImage<Type, Channels> & operator=(const QVImage<sFloat, 3> &sourceImage);
		
		/// @brief Equality compare operator
		///
		/// This operator will compare two images of same bit depth and channel number, and will return true if
		/// images are equal in all of these:
		///	- dimensions (cols and rows).
		///	- region of interest.
		///	- content of region of interest.
		/// returning false otherwise.
		///
		/// @param img image to compare with this image.
		bool operator==(const QVImage<Type, Channels> &img) const;

		/// @brief Inequality compare operator
		///
		/// This operator will compare two images of same bit depth and channel number, and will return true if
		/// images differ in any of:
		///	- dimensions (cols and rows).
		///	- region of interest.
		///	- content of region of interest.
		/// returning false otherwise.
		///
		/// \todo Shouldn't ignore dimensions in caparisons, and only check if region of interest content are the same?
		///
		/// @param img image to compare with this image.
		/// @returns true if images are practically equal (same content of region of interest and dimensions).
		bool operator!=(const QVImage<Type, Channels> &img) const { return !(*this == img); }

		/// @brief Pixel wise 'less than' compare operator
		///
		/// @param img image to compare with this image.
		/// @returns 1-channel uChar bit depth image containing non-zero values for pixels which value for corresponding
		/// pixel in this image is less than value for corresponding pixel in given image.
		QVImage<uChar> operator<(const QVImage<Type, Channels> &img) const;

		/// @brief Pixel wise 'greater than' compare operator
		///
		/// @param img image to compare with this image.
		/// @returns 1-channel uChar bit depth image containing non-zero values for pixels which value for corresponding
		/// pixel in this image is greater than value for corresponding pixel in given image.
		QVImage<uChar> operator>(const QVImage<Type, Channels> &img) const;

		/// @brief Pixel wise 'less or equal than' compare operator
		///
		/// @param img image to compare with this image.
		/// @returns 1-channel uChar bit depth image containing non-zero values for pixels which value for corresponding
		/// pixel in this image is less or equal than value for corresponding pixel in given image.
		QVImage<uChar> operator<=(const QVImage<Type, Channels> &img) const;

		/// @brief Pixel wise 'greater or equal than' compare operator
		///
		/// @param img image to compare with this image.
		/// @returns 1-channel uChar bit depth image containing non-zero values for pixels which value for corresponding
		/// pixel in this image is greater or equal than value for corresponding pixel in given image.
		QVImage<uChar> operator>=(const QVImage<Type, Channels> &img) const;

		/// @brief Pixel wise constant add operator.
		///
		/// Uses as the second operand a constant value for every pixel computed.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param constant second operand value.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator+(const Type constant) const;

		/// @brief Pixel wise constant product operator.
		///
		/// Uses as the second operand a constant value for every pixel computed.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param constant second operand value.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator*(const Type constant) const;

		/// @brief Pixel wise constant subtraction operator.
		///
		/// Uses as the second operand a constant value for every pixel computed.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param constant second operand value.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator-(const Type constant) const;

		/// @brief Pixel wise constant division operator.
		///
		/// Uses as the second operand a constant value for every pixel computed.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param constant second operand value.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator/(const Type constant) const;

		/// @brief Pixel wise bit-wise left shift operator.
		///
		/// Uses as the value to shift a constant value for every pixel computed.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param constant second operand value.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator<<(const Type constant) const;

		/// @brief Pixel wise bit-wise right shift operator.
		///
		/// Uses as the value to shift a constant value for every pixel computed.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param constant second operand value.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator>>(const Type constant) const;

		/// @brief Pixel wise bit-wise NOT operation.
		///
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator!() const;

		/// @brief Pixel wise bit-wise boolean and operator.
		///
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param constant second operand value.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator&(const Type constant) const;

		/// @brief Pixel wise bit-wise boolean or operator.
		///
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param constant second operand value.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator|(const Type constant) const;

		/// @brief Pixel wise bit-wise boolean xor operator.
		///
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param constant second operand value.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator^(const Type constant) const;

		/// @brief Pixel wise add operator.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param img operand image.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator+(const QVImage<Type, Channels> &img) const;

		/// @brief Pixel wise mult operator.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param img operand image.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator*(const QVImage<Type, Channels> &img) const;

		/// @brief Pixel wise subtraction operator.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param img operand image.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator-(const QVImage<Type, Channels> &img) const;

		/// @brief Pixel wise division operator.
		///
		/// Saturation is used for integer bit depth data (any except sFloat bit depth).
		/// Regions of interests, and anchors are applied in this operator.
		///
		/// @param img operand image.
		/// @returns resulting image for the operation.
		QVImage<Type, Channels> operator/(const QVImage<Type, Channels> &img) const;

	protected:
		QSharedDataPointer< QVImageBuffer<Type,Channels> > imageBuffer;
	};

template <typename Type, int C> bool QVImage<Type, C>::operator==(const QVImage<Type, C> &img) const
	{
	Q_ASSERT_X(img.getChannels() == this->getChannels(), "QVImage::operator==", "different number of planes");
	if (this->getCols() != img.getCols()) return false;
	if (this->getRows() != img.getRows()) return false;
	if (this->getChannels() != img.getChannels()) return false;
	if (this->getROI() != img.getROI()) return false;
	QVIMAGE_INIT_READ(Type,img);
	QVIMAGE_PTR_INIT_READ(Type,this);

	uInt x0 = (uInt) img.getROI().x(), y0 = (uInt) img.getROI().y(); 
	uInt x1 = (uInt) img.getROI().width(), y1 = (uInt) img.getROI().height(); 

	uInt lineSize = x1 * img.getChannels();
	for(uInt row = y0; row < y1; row++)
		if (memcmp(&QVIMAGE_PIXEL(img, x0, row, 0),&QVIMAGE_PIXEL(this, x0, row, 0), lineSize) != 0)
			return false;	
	return true;
	};


typedef QVImage<uChar,1> QVImageUCharC1;
typedef QVImage<uChar,3> QVImageUCharC3;
typedef QVImage<sFloat,1> QVImageSFloatC1;
typedef QVImage<sFloat,3> QVImageSFloatC3;
typedef QVImage<sShort,1> QVImageSShortC1;
typedef QVImage<sShort,3> QVImageSShortC3;

//Q_DECLARE_METATYPE(QVGenericImage);
Q_DECLARE_METATYPE(QVImageUCharC1);
Q_DECLARE_METATYPE(QVImageUCharC3);
Q_DECLARE_METATYPE(QVImageSFloatC1);
Q_DECLARE_METATYPE(QVImageSFloatC3);
Q_DECLARE_METATYPE(QVImageSShortC1);
Q_DECLARE_METATYPE(QVImageSShortC3);

#endif // QVIMAGE_H
