/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvdta.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVDTA_H
#define QVDTA_H

#include <QList>
#include <QMap>
#include <QVector>
#include <QPoint>
#include <qvdta/qvdisjointset.h>
#include <qvdta/qvpolyline.h>

/*!
@brief Computer Vision Data types and Algorithms.

See \ref Structure for a detailed explanation of the structure of this package.

*/

/// @brief Sorts pixels in an image, given their gray-scale value.
/// @ingroup qvdta
///
/// This function obtains a QVector, that maps any gray-scale values (0..255) to the set of QPoints that hold that gray-scale
/// value, in a given image. It can be used for very fast pixel sorting.
///
/// The function itself is based in \htmlonly <a href="http://en.wikipedia.org/wiki/Counting_sort">Counting Sort algorithm</a> \endhtmlonly, so the cost of this function is O(number_of_columns * number_of_rows).
///
/// @param img source image to obtain sorted lists of QPoints.
/// @return A QVector, indexed by the gray-scale value (0-255), to a QVector of QPoints, containing the corresponding points to the pixels that hold that gray-scale value in the image.
QVector< QVector< QPoint > > CountingSort(const QVImage<uChar, 1> &img);

/// @brief Generates a binary image of local grey-scale maximal value pixels.
/// @ingroup qvdta
///
/// This function receives a QVImage, and generates a binary image where each pixel is set to IPP_MAX_8U if the pixel in the
/// original image is strict maximal in value regarding to pixels in a vicinity window of colMaskSize width, and rowMaskSize
/// height.
///
/// \todo
///	- Fix resulting image ROI, obtain maximums restricted to the ROI.
///	- Use IPP functions to obtain the result.
///
/// @param src source image.
/// @param dest binary image that will contain maximal values.
/// @param colMaskSize width of the vicinity.
/// @param rowMaskSize height of the vicinity.
void FilterLocalMax(const QVImage<sFloat> &src, QVImage<uChar> &dest, uInt colMaskSize, uInt rowMaskSize, sFloat threshold = 0);

/// @ingroup qvdta
void HarrisCornerResponseImage(const QVImage<uChar> &image, QVImage<sFloat> &result);

/// @ingroup qvdta
void SobelCornerResponseImage(const QVImage<uChar> &image, QVImage<sFloat> &result);

/// @ingroup qvdta
int myFloodFill(QVImage<uChar> &image, uInt x, uInt y, uInt value, uInt minVal, uInt maxVal);

/// @ingroup qvdta
void equalizeHistogram(const QVImage<uChar,1> &image, QVImage<uChar,1> &equalized);

/// @ingroup qvdta
void equalizeHistogram(const QVImage<sFloat,1> &image, QVImage<sFloat,1> &equalized);

#endif
